package myPackage;

import myPackage.objects.ContextItem;

import javax.naming.directory.*;
import javax.naming.NamingException;
import javax.naming.Context;
import javax.naming.Binding;
import javax.naming.NamingEnumeration;
import javax.swing.*;
import javax.swing.tree.DefaultMutableTreeNode;
import java.util.Hashtable;
import java.util.ArrayList;

/**
 * Created by IntelliJ IDEA.
 * User: ls
 * Date: 22.01.2006
 * Time: 16:50:46
 * To change this template use File | Settings | File Templates.
 */
public class Ldap
{
    private final String user_dn   = "cn=InfoPointAdmin,o=InfoPoint";
    private final String password  = "secret";
    private final String host      = "ldap://127.0.0.1";
    private final int port         = 389;
    private final String base_dn   = "o=InfoPoint";

    private InitialDirContext rootContext = null;


    public static void main(String[] args)
    {
        Ldap main = null;

        try
        {
            main = new Ldap();

            
            main.listNodes("");

            /*
            main.getObjectClasses("");

            main.getAttributes("", null);

            JTree tree = main.getTree("ou=Abtl.Entwicklung");
            tree.toString();
            */

            /*
            ContextItem[] items = main.search(new String[] {"url", "displayName"}, "", "person", false);

            if (items != null)
            {
                for (int i = 0; i < items.length; i++)
                {
                    ContextItem item = items[i];
                    System.out.print("DN: " + item.getDirContext().getNameInNamespace() + "   ");

                    Attribute[] attrs = item.getAttributes();

                    for (int j = 0; j < attrs.length; j++)
                    {
                        Object obj = main.getAttributeValue(attrs[j]);
                        System.out.print(obj + "  ");
                    }

                    System.out.print("\n");
                }
            }
            else
            {
                System.out.println("Kein Ergebnis");
            }
            */
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            main.close();
        }
    }

    /**
     * Standard: Konstruktor
     * Initalisiert eine LDAP-Connection
     * @throws NamingException
     */
    public Ldap() throws NamingException
    {
        String  provider_url = host + ":" + port + "/" + base_dn;

        // Angaben zur Verbindung
        Hashtable env = new Hashtable();
        env.put(Context.SECURITY_AUTHENTICATION, "simple");
        env.put(Context.SECURITY_PRINCIPAL, user_dn); // User
        env.put(Context.SECURITY_CREDENTIALS, password); // Password
        env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
        env.put(Context.PROVIDER_URL, provider_url);

        // Verbindungsaufbau
        rootContext = new InitialDirContext(env);
    }

    /**
     * Schliesst die offene LDAP-Connection
     */
    public void close()
    {
        try
        {
            if (rootContext != null)
            {
                rootContext.close();
                System.out.println("Connection closed ...");
            }
        }
        catch (NamingException e)
        {
            e.printStackTrace();
        }
    }

    /**
     * Gibt die Base-DN zurck
     * @return DirContext
     */
    public DirContext getRootContext()
    {
        return rootContext;
    }

    /**
     * Listet alle Knoten ab einer DN auf.
     * @param dn
     * @return
     * @throws NamingException
     */
    public DirContext[] listNodes(String dn) throws NamingException
    {
        //NamingEnumeration list = rootContext.listBindings(""); // Auflistung von Knoten ab der Base DN
        //NamingEnumeration list = rootContext.listBindings("ou=Abtl.Entwicklung"); // Auflistung von Knoten ab einem Unterknoten (Base DN nicht angeben)
        NamingEnumeration list = rootContext.listBindings(dn);

        ArrayList values = new ArrayList();
        while (list.hasMore())
        {
            Binding binding = (Binding)list.next();
            DirContext context = (DirContext)binding.getObject();

            //System.out.println(binding.getName() + " / " + binding.getNameInNamespace());

            values.add(context);
        }


        if (values.size() > 0)
        {
            DirContext[] data = new DirContext[values.size()];
            for (int i = 0; i < values.size(); i++)
            {
                data[i] = (DirContext)values.get(i);
            }

            return data;
        }
        else
            return null;
    }

    /**
     * Gibt die objectClass-Attribute zurck, welche dem Knoten zugeordnet sind.
     * @param dn
     * @return
     * @throws NamingException
     */
    public String[] getObjectClasses(String dn) throws NamingException
    {
        //DirContext context = rootContext.getSchemaClassDefinition(""); // objectClass von Base DN
        //DirContext context = rootContext.getSchemaClassDefinition("ou=Abtl.Entwicklung"); // objectClass von einem Unterknoten (Base DN nicht angeben)
        DirContext context = rootContext.getSchemaClassDefinition(dn);

        NamingEnumeration list = context.listBindings("");

        ArrayList values = new ArrayList();
        while (list.hasMore())
        {
            Binding binding = (Binding)list.next();
            String name = binding.getName();

            //System.out.println(name);

            values.add(name);
        }


        if (values.size() > 0)
        {
            String[] data = new String[values.size()];
            for (int i = 0; i < values.size(); i++)
            {
                data[i] = (String)values.get(i);
            }

            return data;
        }
        else
            return null;
    }

    /**
     * Gibt alle Attribute zurck, welche dem Knoten zugeordnet sind. Mit dem 2. Parameter kann angegeben, welche Attribute
     * zurckgegeben werden sollen.
     * @param dn
     * @return
     * @throws NamingException
     */
    public Attribute[] getAttributes(String dn, String[] attrIds) throws NamingException
    {
        Attributes attrs;
        if (attrIds == null)
            attrs = rootContext.getAttributes(dn);
        else
            attrs = rootContext.getAttributes(dn, attrIds);

        NamingEnumeration enumAttr = attrs.getIDs();

        ArrayList values = new ArrayList();
        while (enumAttr.hasMore())
        {
            String name = (String)enumAttr.next();
            Attribute attr = attrs.get(name);

            //System.out.println(attr.getID());

            values.add(attr);
        }


        if (values.size() > 0)
        {
            Attribute[] array = new Attribute[values.size()];
            for (int i = 0; i < values.size(); i++)
            {
                array[i] = (Attribute)values.get(i);
            }

            return array;
        }
        else
            return null;
    }

    /**
     * Gibt zu einem Attribut seinen Wert zurck.
     * @param attr
     * @return
     * @throws NamingException
     */
    public Object getAttributeValue(Attribute attr) throws NamingException
    {
        return attr.get();
    }

    public JTree getTree(String dn) throws Exception
    {
        System.out.println("Aufbau des Baumes ...");

        if (rootContext != null)
        {
            Attribute[] attrs = this.getAttributes(dn, null);

            DirContext context = (DirContext)rootContext.lookup(dn);

            ContextItem contextItem = new ContextItem(context, attrs);
            DefaultMutableTreeNode root = new DefaultMutableTreeNode(contextItem);

            ArrayList abzusuchende_knoten = new ArrayList();
            abzusuchende_knoten.add(root);

            // Baum aufbauen
            while (!abzusuchende_knoten.isEmpty())
            {
                // abzuarbeitender Knoten bestimmen und Knoten aus ArrayList nehmen
                DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode)abzusuchende_knoten.get(0);
                abzusuchende_knoten.remove(0);
                contextItem = (ContextItem)currentNode.getUserObject();

                // Kinder bestimmen (mit SQL-Statement)
                ContextItem[] children = this.getChildren(rootContext, contextItem);

                if (children.length > 0)
                {
                    for (int i = 0; i < children.length; i++)
                    {
                        contextItem = children[i];

                        // Hier knnen weitere SQL ausgefhren werden

                        DefaultMutableTreeNode knoten = new DefaultMutableTreeNode(contextItem);

                        // Kinderknoten dem aktuellen Knoten anhngen
                        currentNode.add(knoten);

                        // Kinderknoten in abzusuchende_knoten legen
                        abzusuchende_knoten.add(knoten);
                    }
                }
            }

            return new JTree(root);
        }

        return null;
    }

    private ContextItem[] getChildren(DirContext rootContext, ContextItem contextItem) throws NamingException
    {
        ArrayList data = new ArrayList();

        String dn = contextItem.getDirContext().getNameInNamespace();
        dn = this.prepareDN(dn);

        DirContext[] dirs = this.listNodes(dn);

        if (dirs != null)
        {
            for (int i = 0; i < dirs.length; i++)
            {
                DirContext dir = dirs[i];

                dn = dir.getNameInNamespace();
                dn = this.prepareDN(dn);

                Attribute[] attrs = this.getAttributes(dn, null);

                ContextItem temp = new ContextItem(dir, attrs);
                data.add(temp);
            }
        }

        // umwandeln
        ContextItem[] items = new ContextItem[data.size()];
        for (int i = 0; i < items.length; i++)
        {
            items[i] = (ContextItem)data.get(i);
        }

        return items;
    }

    private String prepareDN(String dn)
    {
        dn = dn.replaceAll(base_dn, "");

        // Komma an letzter Stelle entfernen
        int length = dn.length();

        if (length > 0)
        {
            if (dn.lastIndexOf(",") == (length - 1))
                dn = dn.substring(0, (length - 1));
        }

        return dn;
    }

    public ContextItem[] search(String[] attrIds, String dn, String objectClass, boolean oneLevelScope) throws Exception
    {
        if (attrIds == null)
            throw new Exception("Kein Attribut-Id gefunden");

        SearchControls ctls = new SearchControls();

        // Such-Tiefe
        if (oneLevelScope)
            ctls.setSearchScope(SearchControls.ONELEVEL_SCOPE);
        else
            ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);

        ctls.setReturningObjFlag(true);

        // Rckgabe von Attributen
        ctls.setReturningAttributes(attrIds);

        // objectClass-Filter
        String filter = "(objectClass=*)";
        if (objectClass != null)
            filter = "(objectClass=" + objectClass + ")";


        NamingEnumeration enumSearchResult = rootContext.search(dn, filter, ctls);

        ArrayList data = new ArrayList();
        while (enumSearchResult.hasMore())
        {
            SearchResult result = (SearchResult)enumSearchResult.next();
            Attributes attrs = result.getAttributes();

            DirContext userCtx = (DirContext)result.getObject();

            NamingEnumeration enumAttr = attrs.getIDs();

            ArrayList values = new ArrayList();
            while (enumAttr.hasMore())
            {
                String name = (String)enumAttr.next();
                Attribute attr = attrs.get(name);

                values.add(attr);
            }


            Attribute[] array = null;
            if (values.size() > 0)
            {
                array = new Attribute[values.size()];
                for (int i = 0; i < values.size(); i++)
                {
                    array[i] = (Attribute)values.get(i);
                }
            }

            ContextItem item = new ContextItem(userCtx, array);
            data.add(item);
        }


        if (data.size() > 0)
        {
            ContextItem[] items = new ContextItem[data.size()];
            for (int i = 0; i < data.size(); i++)
            {
                items[i] = (ContextItem)data.get(i);
            }

            return items;
        }

        return null;
    }
}
